/**
 * EasyBeans
 * Copyright (C) 2006-2008 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id:CarolComponent.java 1477 2007-06-16 16:50:19Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.ow2.easybeans.component.carol;

import java.util.ArrayList;
import java.util.List;
import java.util.Properties;

import javax.management.MBeanServer;
import javax.naming.Context;

import org.objectweb.jotm.jta.rmi.JTAInterceptorInitializer;
import org.ow2.carol.jndi.ns.NameServiceManager;
import org.ow2.carol.util.configuration.ConfigurationRepository;
import org.ow2.easybeans.component.api.EZBComponentException;
import org.ow2.easybeans.component.itf.RegistryComponent;
import org.ow2.easybeans.component.util.Property;
import org.ow2.easybeans.jmx.JMXRemoteException;
import org.ow2.easybeans.jmx.MBeanServerHelper;
import org.ow2.easybeans.security.propagation.rmi.jrmp.interceptors.SecurityInitializer;
import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;

/**
 * This component allows to start a registry.
 * @author Florent Benoit
 */
public class CarolComponent implements RegistryComponent {

    /**
     * Logger.
     */
    private Log logger = LogFactory.getLog(CarolComponent.class);

    /**
     * Default prefix protocol.
     */
    private static final String DEFAULT_PREFIX_PROTOCOL = "rmi";

    /**
     * List of protocols.
     */
    private List<Protocol> protocols = null;

    /**
     * Global Initial Context Factory.
     */
    private String initialContextFactory = null;

    /**
     * Additional properties for Carol.
     */
    private List<Property> carolProperties = null;

    /**
     * Creates a new Carol component.
     */
    public CarolComponent() {
        this.protocols = new ArrayList<Protocol>();
    }

    /**
     * Init method.<br/>
     * This method is called before the start method.
     * @throws EZBComponentException if the initialization has failed.
     */
    @SuppressWarnings("deprecation")
    public void init() throws EZBComponentException {
        // check
        if (this.protocols == null || this.protocols.isEmpty()) {
            this.logger.debug("No protocols, use the existing carol configuration");
            return;
        }

        // Get Value of the Initial Factory
        this.initialContextFactory = System.getProperty(Context.INITIAL_CONTEXT_FACTORY);

        // List of protocols
        String lstProtocol = null;

        // Create configuration
        Properties carolConf = new Properties();

        // Set carol mode in server mode (will use fixed port if set)
        System.setProperty("carol.server.mode", "true");


        if(System.getProperty(Context.URL_PKG_PREFIXES) == null) {
        // Use EZB url package factory
        System.setProperty(Context.URL_PKG_PREFIXES, "org.ow2.easybeans.naming.pkg");
        }

        // Use a local access to registry for optimization
        carolConf.setProperty("carol.jvm.rmi.local.registry", "true");

        for (Protocol protocol : this.protocols) {

            String protocolName = protocol.getName();

            // Build list of protocols
            if (lstProtocol != null) {
                lstProtocol += ",";
                lstProtocol += protocolName;
            } else {
                lstProtocol = protocolName;
            }

            //Define URL
            String url;
            if (protocol.getUrl() != null) {
                // set defined URL
                url = protocol.getUrl();
            } else {
                // compute URL
                String host = protocol.getHostname();
                int portNumber = protocol.getPortNumber();
                url = DEFAULT_PREFIX_PROTOCOL + "://" + host + ":" + portNumber;
            }
            carolConf.setProperty("carol." + protocolName + ".url", url);
        }

        carolConf.setProperty("carol.protocols", lstProtocol);

        // Get additional properties (override the previous, should this happen)
        if (this.carolProperties != null) {
            for (Property property : this.carolProperties) {
                carolConf.setProperty(property.getName(), property.getValue());
            }
        }

        this.logger.debug("carolProps: {0}", carolConf);

        String domainName = null;
        try {
            MBeanServer mBeanServer = MBeanServerHelper.getMBeanServerServer();
            if (mBeanServer != null) {
                domainName = mBeanServer.getDefaultDomain();
            }
        } catch (JMXRemoteException e) {
            this.logger.debug("No domain name for JMX", e);
        }

        try {
            ConfigurationRepository.init(carolConf, domainName, "EasyBeans");
        } catch (Exception e) {
            throw new EZBComponentException("Cannot initialize registry", e);
        }

        try {
            ConfigurationRepository.addInterceptors("jrmp", JTAInterceptorInitializer.class);
        } catch (Exception e) {
            throw new EZBComponentException("Cannot add JOTM interceptors", e);
        }

        try {
            ConfigurationRepository.addInterceptors("iiop", "org.objectweb.jotm.ots.OTSORBInitializer");
        } catch (Exception e) {
            throw new EZBComponentException("Cannot add JOTM interceptors", e);
        }

        try {
            ConfigurationRepository.addInterceptors("jrmp", SecurityInitializer.class);
        } catch (Exception e) {
            throw new EZBComponentException("Cannot add Security interceptors", e);
        }

    }


    /**
     * Start method.<br/>
     * This method is called after the init method.
     * @throws EZBComponentException if the start has failed.
     */
    public void start() throws EZBComponentException {
        // Start registry only if there are protocols
        if (this.protocols != null && !this.protocols.isEmpty()) {
            try {
                NameServiceManager.getNameServiceManager().startNS();
            } catch (Exception e) {
                throw new EZBComponentException("Cannot start registry", e);
            }
        }
    }


    /**
     * Stop method.<br/>
     * This method is called when component needs to be stopped.
     * @throws EZBComponentException if the stop is failing.
     */
    public void stop() throws EZBComponentException {
        // Stop registry only if there are protocols
        if (this.protocols != null && !this.protocols.isEmpty()) {
            try {
                NameServiceManager.getNameServiceManager().stopNS();
            } catch (Exception e) {
                throw new EZBComponentException("Cannot stop the registry", e);
            }

            // Restore previous value of the Initial Context Factory
            if (this.initialContextFactory != null) {
                System.setProperty(Context.INITIAL_CONTEXT_FACTORY, this.initialContextFactory);
            } else {
                // Remove property
                System.getProperties().remove(Context.INITIAL_CONTEXT_FACTORY);
            }

        }
        this.logger.info("Carol Component Stopped");
    }

    /**
     * Gets the protocols defined for the start.
     * @return the list of protocols.
     */
    public List<Protocol> getProtocols() {
        return this.protocols;
    }

    /**
     * Sets the list of protocols.
     * @param protocols the list of protocols configured for this server.
     */
    public void setProtocols(final List<Protocol> protocols) {
        this.protocols = protocols;
    }

    /**
     * Gets the default Provider URL.
     * Note: The old API needs to be used here in order to be compliant with products using Carol v2.x
     * @return the provider URL that is used by default.
     */
    public String getProviderURL() {
        return org.objectweb.carol.util.configuration.ConfigurationRepository.getCurrentConfiguration().getProviderURL();
    }

    /**
     * Gets the list of properties.
     * @return the list of properties.
     */
    public List<Property> getProperties() {
        return this.carolProperties;
    }

    /**
     * Set the list of properties.
     * @param carolProperties the list of properties.
     */
    public void setProperties(final List<Property> carolProperties) {
        this.carolProperties = carolProperties;
    }


}
