/**
 * EasyBeans
 * Copyright (C) 2007,2008 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: CmiComponent.java 3585 2008-07-02 07:29:13Z eyindanga $
 * --------------------------------------------------------------------------
 */

package org.ow2.easybeans.component.cmi;

import java.util.Iterator;
import java.util.List;
import java.util.Properties;

import org.ow2.carol.util.configuration.ConfigurationRepository;
import org.ow2.cmi.controller.server.AbsServerClusterViewManager;
import org.ow2.cmi.controller.server.ServerClusterViewManager;
import org.ow2.easybeans.component.api.EZBComponent;
import org.ow2.easybeans.component.api.EZBComponentException;
import org.ow2.easybeans.server.ServerConfig;
import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;

/**
 * Component providing a support of CMI.
 * @author WEI Zhouyue & ZHU Ning
 */
public class CmiComponent implements EZBComponent {

    /**
     * Logger.
     */
    private Log logger = LogFactory.getLog(CmiComponent.class);

    /**
     * Configuration of the embedded instance.
     */
    private ServerConfig config;

    /**
     * True if this component manages the lifecycle of the cluster view manager.
     */
    private boolean lifecycleManaged = true;

    /**
     * The manager of cluster view.
     */
    private ServerClusterViewManager clusterViewManager;

    /**
     * CMI mode.
     */
    private String mode = CmiComponentProperty.SERVER_MODE_VALUE.getPropertyName();

    /**
     * List of protocols that are enabled for CMI.
     */
    private List<String> protocols = null;

    /**
     * Some default properties.
     */
    private Properties cmiProperties = new Properties();

    /**
     * CMI initial context factory.
     */
    private String contextFactory = CmiComponentProperty.DEFAULT_CMI_INITIAL_CONTEXT_FACTORY.getPropertyName();

    /**
     * Setter for CMI properties.
     */
    private String cmiPropertiesSetter = CmiComponentProperty.DEFAULT_CMI_PROPERTIES_SETTER_METHOD.getPropertyName();

    /**
     * Setter for CMI Mbean.
     */
    private Object cmiMBeanSetter = CmiComponentProperty.DEFAULT_CMI_MBEAN_SETTER_METHOD.getPropertyName();

    /**
     * True if replication is enabled.
     */
    private boolean replicationEnabled = true;


    /**
     * @return the protocols
     */
    public List<String> getProtocols() {
        return this.protocols;
    }

    /**
     * @param protocols the protocols to set
     */
    public void setProtocols(final List<String> protocols) {
        this.protocols = protocols;
    }

    /**
     * Init method.<br/> This method is called before the start method.
     * @throws EZBComponentException if the initialization has failed.
     */
    public void init() throws EZBComponentException {
        initProperties();
    }

    /**
     * Initilizes user defined properties.
     */
    private void initProperties() {
        // TODO Auto-generated method stub
        this.cmiProperties.put(CmiComponentProperty.SERVER_MODE_KEY.getPropertyName(), this.mode
                .equalsIgnoreCase(CmiComponentProperty.SERVER_MODE_VALUE.getPropertyName()));
        this.cmiProperties.put(CmiComponentProperty.CMI_INITIAL_CONTEXT_FACTORY_KEY.getPropertyName(), this.contextFactory);
        this.cmiProperties.put(CmiComponentProperty.CMI_PROPERTIES_SETTER_METHOD_KEY.getPropertyName(),
                this.cmiPropertiesSetter);
        this.cmiProperties.put(CmiComponentProperty.CMI_MBEAN_SETTER_METHOD_KEY.getPropertyName(), this.cmiMBeanSetter);
        this.cmiProperties.put(CmiComponentProperty.CMI_ENABLE_REPLICATION_KEY.getPropertyName(), this.replicationEnabled);
        if (this.protocols != null) {
            for (Iterator<String> iterator = this.protocols.iterator(); iterator.hasNext();) {
                String protocol = iterator.next();
                String key = CmiComponentProperty.CMI.getPropertyName() + CmiComponentProperty.SEPARATOR.getPropertyName()
                        + protocol + CmiComponentProperty.SEPARATOR.getPropertyName()
                        + CmiComponentProperty.ENABLE.getPropertyName();
                this.cmiProperties.put(key, true);
            }
        }


    }

    /**
     * Start method.<br/> This method is called after the init method.
     * @throws EZBComponentException if the start has failed.
     */
    public void start() throws EZBComponentException {
        if (this.lifecycleManaged) {
            // Enable bindings into the cluster
            try {
                ConfigurationRepository.getServerConfiguration().enableCMI(this.cmiProperties);
            } catch (Exception e) {
                this.logger.error("Cannot configure Carol to use CMI", e);
                throw new EZBComponentException("Cannot configure Carol to use CMI", e);
            }

            // Start the manager
            try {
                this.clusterViewManager = AbsServerClusterViewManager.start();
            } catch (Exception e) {
                this.logger.error("Cannot start the server-side manager", e);
                throw new EZBComponentException("Cannot start the server-side manager", e);
            }
        }
        // Add the callback
        this.config.addExtensionFactory(new CmiConfigurationExtension());
        this.logger.debug("The CMI configuration extension has been added.");
    }

    /**
     * Stop method.<br/> This method is called when component needs to be
     * stopped.
     * @throws EZBComponentException if the stop is failing.
     */
    public void stop() throws EZBComponentException {
        if (this.lifecycleManaged) {
            try {
                ConfigurationRepository.getServerConfiguration().disableCMI();
            } catch (Exception e) {
                this.logger.error("Cannot disable CMI in Carol", e);
            }
            if (this.clusterViewManager != null) {
                try {
                    ((AbsServerClusterViewManager) this.clusterViewManager).stop();
                } catch (Exception e) {
                    this.logger.error("Cannot stop the server-side manager", e);
                    throw new EZBComponentException("Cannot stop the server-side manager", e);
                }
            }
        }
    }

    /**
     * Return the configuration of the embedded instance.
     * @return the configuration of the embedded instance
     */
    public ServerConfig getServerConfig() {
        return this.config;
    }

    /**
     * Set the configuration of the embedded instance.
     * @param config the configuration of the embedded instance
     */
    public void setServerConfig(final ServerConfig config) {
        this.config = config;
    }

    /**
     * @return true if this component manages the lifecycle of the cluster view
     *         manager
     */
    public boolean isLifecycleManaged() {
        return this.lifecycleManaged;
    }

    /**
     * Set if this component manages the lifecycle of the cluster view manager.
     * @param lifecycleManaged true if this component manages the lifecycle of
     *        the cluster view manager
     */
    public void setLifecycleManaged(final boolean lifecycleManaged) {
        this.lifecycleManaged = lifecycleManaged;
    }

    /**
     * @return the mode.
     */
    public String getMode() {
        return this.mode;
    }

    /**
     * @param mode the mode to set.
     */
    public void setMode(final String mode) {
        this.mode = mode;
    }

    /**
     * @return the clusterViewManager.
     */
    public ServerClusterViewManager getClusterViewManager() {
        return this.clusterViewManager;
    }

    /**
     * @param clusterViewManager the clusterViewManager to set.
     */
    public void setClusterViewManager(final ServerClusterViewManager clusterViewManager) {
        this.clusterViewManager = clusterViewManager;
    }

    /**
     * @return the contextFactory.
     */
    public String getInitialContextFactory() {
        return this.contextFactory;
    }

    /**
     * @param contextFactory the contextFactory to set.
     */
    public void setInitialContextFactory(final String contextFactory) {
        this.contextFactory = contextFactory;
    }

    /**
     * @return the cmiPropertiesSetter.
     */
    public String getCmiPropertiesSetter() {
        return this.cmiPropertiesSetter;
    }

    /**
     * @param cmiPropertiesSetter the cmiPropertiesSetter to set.
     */
    public void setCmiPropertiesSetter(final String cmiPropertiesSetter) {
        this.cmiPropertiesSetter = cmiPropertiesSetter;
    }

    /**
     * @return the cmiMBeanSetter.
     */
    public Object getCmiMBeanSetter() {
        return this.cmiMBeanSetter;
    }

    /**
     * @param cmiMBeanSetter the cmiMBeanSetter to set.
     */
    public void setCmiMBeanSetter(final String cmiMBeanSetter) {
        this.cmiMBeanSetter = cmiMBeanSetter;
    }

    /**
     * @param enableReplication
     */
    public void setEnableReplication(final String enableReplication) {
        this.replicationEnabled = enableReplication.equalsIgnoreCase("true");

    }

    /**
     * @return
     */
    public boolean isReplicationEnabled() {
        return this.replicationEnabled;
    }

}
