/**
 * EasyBeans
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: SecurityInfo.java 1970 2007-10-16 11:49:25Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.ow2.easybeans.container.info.security;

import java.security.Principal;
import java.security.acl.Group;
import java.util.ArrayList;
import java.util.List;

import javax.security.auth.Subject;

import org.ow2.easybeans.api.bean.info.IMethodSecurityInfo;
import org.ow2.easybeans.api.bean.info.ISecurityInfo;
import org.ow2.easybeans.security.struct.JGroup;
import org.ow2.easybeans.security.struct.JPrincipal;

/**
 * Runtime info about security.
 * @author Florent Benoit
 */
public class SecurityInfo implements ISecurityInfo {

    /**
     * List of roles.
     */
    private List<String> declaredRoles = null;

    /**
     * List of methods.
     */
    private List<IMethodSecurityInfo> methodSecurityInfos = null;

    /**
     * Name of the run-as role.
     */
    private String runAsRole = null;

    /**
     * Subject for run-as role.
     */
    private Subject runAsSubject = null;

    /**
     * Default constructor.
     */
    public SecurityInfo() {
        this.methodSecurityInfos = new ArrayList<IMethodSecurityInfo>();
    }

    /**
     * Adds a method containing security.
     * @param methodSecurityInfo the info about security.
     */
    public void addMethodSecurityInfo(final IMethodSecurityInfo methodSecurityInfo) {
        methodSecurityInfos.add(methodSecurityInfo);
    }

    /**
     * @return list of security infos on all methods.
     */
    public List<IMethodSecurityInfo> getMethodSecurityInfos() {
        return methodSecurityInfos;
    }

    /**
     * Sets the name of the run-as security role.
     * @param runAsRole the name of the role.
     */
    public void setRunAsRole(final String runAsRole) {
        this.runAsRole = runAsRole;
        this.runAsSubject = new Subject();
        // Add principal name
        Principal principalName = new JPrincipal(runAsRole);
        runAsSubject.getPrincipals().add(principalName);

        // Add roles for this principal
        Group roles = new JGroup("roles");
        roles.addMember(new JPrincipal(runAsRole));
        runAsSubject.getPrincipals().add(roles);

    }

    /**
     * Gets run-as name.
     * @return the name of the security role for the run-as.
     */
    public String getRunAsRole() {
        return runAsRole;
    }

    /**
     * Gets run-as role subject.
     * @return a subject with run-as role as role.
     */
    public Subject getRunAsSubject() {
        return runAsSubject;
    }

    /**
     * Adds a role for this bean (for isCallerInRole).
     * @param roleName the name of a role.
     */
    public void addDeclaredRole(final String roleName) {
        declaredRoles.add(roleName);
    }

    /**
     * @return list of roles declared for this bean.
     */
    public List<String> getDeclaredRoles() {
        return declaredRoles;
    }

    /**
     * Sets the list of declared roles.
     * @param declaredRoles list of declared roles.
     */
    public void setDeclaredRole(final List<String> declaredRoles) {
        this.declaredRoles = declaredRoles;
    }
}
