/**
 * EasyBeans
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id:EjbJarArchiveMetadata.java 2372 2008-02-08 18:18:37Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.ow2.easybeans.deployment.metadata.ejbjar;

import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.ejb.ApplicationException;

import org.ow2.easybeans.deployment.metadata.ejbjar.xml.EasyBeansDD;
import org.ow2.util.ee.deploy.api.deployable.EJB3Deployable;
import org.ow2.util.ee.metadata.common.api.struct.IJInterceptors;
import org.ow2.util.ee.metadata.ejbjar.api.IJClassInterceptor;
import org.ow2.util.ee.metadata.ejbjar.api.InterceptorType;
import org.ow2.util.ee.metadata.ejbjar.api.struct.IJCommonBean;
import org.ow2.util.ee.metadata.ejbjar.impl.struct.JApplicationException;
import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;

/**
 * This class represents the annotation metadata of all classes of an EjbJar
 * file. From this class, we can get metadata of all beans.
 * @author Florent Benoit
 */
public class EjbJarArchiveMetadata {

    /**
     * Logger.
     */
    private static Log logger = LogFactory.getLog(EjbJarArchiveMetadata.class);

    /**
     * List of class annotations metadata.
     */
    private Map<String, EjbJarClassMetadata> classesAnnotationMetadata = null;

    /**
     * Link to the Specific Deployment Descriptor object.
     */
    private EasyBeansDD easybeansDD = null;

    /**
     * List of application exceptions used on this ejb-jar.
     */
    private Map<String, ApplicationException> applicationExceptions = null;

    /**
     * List of default interceptors by type (business or lifecycle).
     */
    private Map<InterceptorType, List<? extends IJClassInterceptor>> defaultInterceptors = null;

    /**
     * List of default interceptors classes.
     */
    private IJInterceptors defaultInterceptorsClasses = null;


    /**
     * Constructor.
     */
    public EjbJarArchiveMetadata() {
        this.classesAnnotationMetadata = new HashMap<String, EjbJarClassMetadata>();
    }

    /**
     * Add annotation metadata for a given class.
     * @param classAnnotationMetadata annotation metadata of a class.
     */
    public void addEjbJarClassMetadata(final EjbJarClassMetadata classAnnotationMetadata) {
        String key = classAnnotationMetadata.getClassName();
        // already exists ?
        if (this.classesAnnotationMetadata.containsKey(key)) {
            String msg = logger.getI18n().getMessage("EjbJarAnnotationMetadata.addClassAnnotationMetadata.alreadyPresent", key);
            logger.debug(msg);
            throw new IllegalStateException(msg);
        }
        this.classesAnnotationMetadata.put(key, classAnnotationMetadata);
    }

    /**
     * Gets the class metadata for the given ejb-name.
     * @return class metadata or null if not found
     * @param ejbName the name of the EJB.
     */
    public EjbJarClassMetadata getEjbJarClassMetadataForEjbName(final String ejbName) {
        for (EjbJarClassMetadata metadata : this.classesAnnotationMetadata.values()) {
            IJCommonBean bean = metadata.getJCommonBean();
            if (bean != null && ejbName.equals(bean.getName())) {
                return metadata;
            }
        }
        return null;
    }

    /**
     * Get class annotation metadata.
     * @param className key of the map of annotations bean.
     * @return Bean annotation metadata of a given name.
     */
    public EjbJarClassMetadata getEjbJarClassMetadata(final String className) {
        return this.classesAnnotationMetadata.get(className);
    }

    /**
     * Get collections of bean annotation metadata.
     * @return collections of bean annotation metadata.
     */
    public Collection<EjbJarClassMetadata> getEjbJarClassMetadataCollection() {
        return this.classesAnnotationMetadata.values();
    }

    /**
     * @return the specific deployment descriptor object.
     */
    public EasyBeansDD getEasyBeansDD() {
        return this.easybeansDD;
    }

    /**
     * Sets the easybeans specific DD deployment descriptor object.
     * @param easybeansDD the specific deployment descriptor object.
     */
    public void setEasyBeansDD(final EasyBeansDD easybeansDD) {
        this.easybeansDD = easybeansDD;
    }

    /**
     * Gets the list of application exceptions defined on this ejb jar metadata.
     * @return the list of application exceptions defined on this ejb jar
     *         metadata.
     */
    public Map<String, ApplicationException> getApplicationExceptions() {
        if (this.applicationExceptions != null) {
            return this.applicationExceptions;
        }

        // compute it
        this.applicationExceptions = new HashMap<String, ApplicationException>();

        // For each class, look if it is an application exception
        for (EjbJarClassMetadata classMetadata : getEjbJarClassMetadataCollection()) {
            ApplicationException appException = classMetadata.getApplicationException();
            // found it then add it in the map.
            if (appException != null) {
                this.applicationExceptions.put(classMetadata.getClassName().replaceAll("/", "."), appException);
            }
        }

        // Add a default application exception (for checked exception)
        this.applicationExceptions.put("DEFAULT", new JApplicationException());

        return this.applicationExceptions;
    }


    /**
     * @return Map&lt;interceptor type &lt;--&gt; List of methods/class corresponding to the interceptor&gt; (interceptor classes)
     * of default interceptors that enhancer will use.
     */
    public Map<InterceptorType, List<? extends IJClassInterceptor>> getDefaultInterceptors() {
        return this.defaultInterceptors;
    }

    /**
     * Sets the list of default interceptors that enhancers will use.<br>
     * These interceptors are defined by XML DD.
     * @param defaultInterceptors list of interceptors that enhancer will use.
     */
    public void setDefaultInterceptors(final Map<InterceptorType, List<? extends IJClassInterceptor>> defaultInterceptors) {
        this.defaultInterceptors = defaultInterceptors;
    }

    /**
     * @return object representing list of default interceptor classes.
     */
    public IJInterceptors getDefaultInterceptorsClasses() {
        return this.defaultInterceptorsClasses;
    }

    /**
     * Sets the object representing the default interceptor classes.
     * @param defaultInterceptorsClasses list of classes
     */
    public void setDefaultInterceptorsClasses(final IJInterceptors defaultInterceptorsClasses) {
        this.defaultInterceptorsClasses = defaultInterceptorsClasses;
    }

    private EJB3Deployable deployable;

    public void setDeployable(final EJB3Deployable deployable) {
        this.deployable = deployable;
    }

    public EJB3Deployable getDeployable() {
        return this.deployable;
    }

}
